import os, sys
sys.path.append(os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

import logging
import signal

import Helper
from src.holon import AbdiConfig
from src.holon.HolonicAgent import HolonicAgent

from src.holon.HolonicAgent import HolonicAgent
# from visual.Visual import Visual
from hearing import Hearing
# from voice.Voice import Voice
from navi import Navigator
# from dialog import DialogSystem
import guide_config
# from hearing.trans import Transcriptionist


if __name__ == '__main__':
    # Helper.init_logging()
    # logging.info('***** Main start *****')
    print('***** Navi start *****')

    def signal_handler(signal, frame):
        print("signal_handler")
    signal.signal(signal.SIGINT, signal_handler)

    cfg = AbdiConfig()
    cfg.mqtt_address = guide_config.mqtt_address
    cfg.mqtt_port = guide_config.mqtt_port
    cfg.mqtt_keepalive = guide_config.mqtt_keepalive
    cfg.mqtt_username = guide_config.mqtt_username
    cfg.mqtt_password = guide_config.mqtt_password
    cfg.log_level = guide_config.log_level
    cfg.log_dir = guide_config.log_dir    
    os.environ["OPENAI_API_KEY"] = guide_config.openai_api_key

    Helper.init_logging(log_dir='tests/guide/_log')

    a = Navigator(cfg)
    a.start()

    # time.sleep(5)
    # a.terminate()



import uuid


WRAPPER_HEAD = "950f7f7ba7c111eea5c4ff9ca9F3fcfd"
# WRAPPER_HEAD = "950f7f7b"
WRAPPER_HEAD_LENGTH = len(WRAPPER_HEAD)
CALLBACK_ID_LENGTH = 32


class PayloadWrapper:
    def __init__(self):
        self.binary_wrapper = BinaryWrapper(self.agent_uuid)
        self.text_wrapper = TextWrapper(self.agent_uuid)
        self.__callbacks = {}
        

    def _get_payload_wrapper(self, payload):
        if not payload or isinstance(payload, str):
            return self.text_wrapper.wrap(payload)
        elif isinstance(payload, bytes) or isinstance(payload, bytearray):
            return self.binary_wrapper.wrap(payload)
        else:
            return None


    def unpack(self, payload):
        wrapper = self._get_payload_wrapper(payload)
        if wrapper:
            payload1, callback_func = wrapper.unpack(payload)
            callback_id = str(uuid.uuid1()).replace('-', '')
            payload1 = wrapper.wrap(payload, callback_id)
            self.__callbacks[callback_id] = topic_callback

        
        
    def wrap(self, payload, topic_wait=None, topic_callback=None):
        if not (topic_wait and topic_callback):
            return payload
                    
        payload1 = payload
        wrapper = self._get_payload_wrapper(payload)
        if wrapper:
            callback_id = str(uuid.uuid1()).replace('-', '')
            payload1 = wrapper.wrap(payload, callback_id)
            self.__callbacks[callback_id] = topic_callback

        return payload1
        
        

class BinaryWrapper:
    def __init__(self, agent_uuid:str):
        self.agent_uuid = agent_uuid.encode('ascii')        
        
        
    def wrap(self, payload, callback_id):
        pass
        

            
class TextWrapper:
    def __init__(self):
        pass
        
        
    def unpack(self, payload) -> (str, str):
        if not payload:
            return payload, None
        
        if payload.startswith(WRAPPER_HEAD):
            content_index = WRAPPER_HEAD_LENGTH + CALLBACK_ID_LENGTH
            callback_id = payload[WRAPPER_HEAD_LENGTH, content_index]
            return payload[content_index], callback_id
        else:
            return payload, None
                
        
    def wrap(self, payload, callback_id) -> str:
        if payload:
            return f"{WRAPPER_HEAD}{callback_id}{payload}"
        else:
            return f"{WRAPPER_HEAD}{callback_id}"
